/**
	forwardsearch.cpp
	Purpose: Implementation of the String Forwardsearch with variants

	@author Thomas Büchler
*/
#include "forwardsearch.hpp"
//most definitions in forwardsearch.hpp

/**
 * if the rank of a marker < rank of the seperater between chrom i and j,
 * then the marker is part of chrom i or previous chromosomes.
 * Previous chromosome can be excluded if we run the for-loop from beginning to end.
 */
std::string chrom_name(unsigned int marker_rank, position_helper & pos_h){
	for(unsigned int i = 0; i < pos_h.seq_seperator.size() - 1 ; i++){
		if(marker_rank <= pos_h.seq_seperator[i]){
			return pos_h.chromosome_names[i];
		}		
	}
	//this should never happen.
	//return statement added because of compiler warning
	return("ERROR[chromosome name to " + std::to_string(marker_rank) + " not found]");
};

/**
 * \brief calculates the pos in the fasta file that relates to posistion pos in jis
 * 
 * \param pos jis position
 * \param rs_next_marker rank of the next marker
 * \param off jis offset to next marker
 * 
 * \return string for the output file  
 */
std::string start_in_jis_print(unsigned int pos, unsigned int rs_next_marker, unsigned int off, position_helper & pos_h){
	using namespace std;
	stringstream out;
	out << chrom_name(rs_next_marker,pos_h) 
			<< '\t' << pos_h.fa_pos[rs_next_marker] - off 
			<< '\t' << pos+1 << '\t';
	return move(out.str());
};
