/**
	io_helper.hpp
	Purpose: Help methods to handle IO

	@author Thomas Büchler
*/
#include <string>
#include <unistd.h> //file test
#include <iostream> //cout
#include <iomanip> //setw
#include <fstream> //ifstream

#include <sdsl/vectors.hpp>

#ifndef IOHELP_H
#define IOHELP_H

#define IOLWIDTH 55


void dotline(){
	std::cout << '\r' << std::setfill('.') << std::setw(IOLWIDTH) << '.' << std::setfill(' ')  << std::endl;
}

void print_row(std::string left, double rigth){
  std::cout << '\r' << left 	<< std::setw(IOLWIDTH-left.length()) << std::setprecision(7) 	<< rigth 	<< std::endl; 
}

void print_row(std::string left, unsigned int rigth){
  std::cout << '\r' << left	<< std::setw(IOLWIDTH-left.length())<< rigth 	<< std::endl; 
}

void print_row(std::string left, std::string rigth){
  std::cout << '\r' << left 	<< std::setw(IOLWIDTH-left.length()) 	<< rigth 	<< std::endl; 
}

void flush_row(std::string left, std::string rigth){
  std::cout << '\r' << left << std::setw(IOLWIDTH-left.length())	<< rigth << std::flush; 
}

/**
 * 
 */
bool file_access(std::string filename){
	if(access((filename).c_str(), F_OK)){
    std::cout << "Error: file '" << filename << "' not found!" << std::endl;
    return true; 
  }
  return false;
};

/**
 * 
 */
template <class T>
bool sdsl_load(std::string filename, T & obj){
	bool error = false;
	
	if(file_access(filename)){
		error = true;			
  }else	if( !sdsl::load_from_file<T>(obj,filename)){
    std::cout << "Error: Object could not be loaded from '" << filename << "'" << std::endl;
    error = true; 
  }
  return error;
};

/**
 * 
 */
bool load_string_from_file(std::string  filename, std::string & buffer){
	if(file_access(filename))
		return true; //error
  
	//load jis from file
	std::ifstream stream; 
	stream.open(filename);
	stream >> buffer; 
	stream.close();
	return false;
};

#endif
