/**
	compressedjia_select.hpp
	Purpose: Compressing JIA with vectors and select support 

	@author Thomas Büchler
*/
#include <vector>

#include "jumpindexarray.hpp"

#include <sdsl/int_vector.hpp>
#include <sdsl/bit_vectors.hpp>

#ifndef C2JI_H
#define C2JI_H

//Problems with empty Vectors -> compression with rank support used

/**
 * (In Master Thesis: 6.4.1 Komprimierung mit select support) 
 * \brief compressed jia, compressed infomation that are needed for the 
 * 
 * interfaces:
 * 
 * targets(int i) returns the jumptargets to the # with number i
 * to_vari(int i) returns if the jump goes to a variation
 * 
 * jumps contains jump targets of all markers
 * beginselect marks at wich entry the jumptargets of the next marker begins * 
 * 
 */
struct compressedjia_select {
  typedef sdsl::int_vector<>::size_type	size_type;
  
  //data  
  sdsl::bit_vector begin;
  sdsl::bit_vector to_var;    
  sdsl::int_vector<0> jumps;    
  sdsl::select_support_mcl<> beginselect;
  
  //constructors  
  /** \brief constructs a compressed jia out of a jia */
    compressedjia_select(const jumpindexarray & jia);   
  /** \brief defaut constructor */
    compressedjia_select();
  
  
  //methods  
  /** \brief returns a vector with the jumptargets*/
  sdsl::int_vector<0> targets(int i);  
  /** \brief returns if the jump goes to a variation*/
  bool to_vari(int i); 
  /** \brief uncompresses cia*/
  jumpindexarray jia();
    
  //serialization
  /** \brief serealizes a jumpindex with sdsls serialize methods */  
  size_type serialize(std::ostream& out, sdsl::structure_tree_node* v, std::string name) const;		
  /** \brief loads a serialized jumpindex with sdsls methods */
  void load(std::istream& in);    
};

std::ostream& operator <<(std::ostream& outputStream, const compressedjia_select& jia);
#endif
