/**
	conflict_group.hpp
	Purpose: Solfing conflicts with overlapping indels in VCF

	@author Thomas Büchler
*/

#include <deque>
#include <string>
#include "vcfl.hpp"

#ifndef CONF_H
#define CONF_H

/** 
 * (In Master Thesis: 6.6 Konflikt bei �berschneidungen) 
 *\brief struct that contains several vcf lines that are in conflict
 */
struct conflict_group{
  //data
  /** lines that are in conflict */
  std::deque<vcfl>	vcf_lines;
  /** last position  that is in conflict*/
  unsigned int maxend;
  
  /** \brief defalut constructor */
  conflict_group();
  
  //methods  
  /** \brief check if line is in conflict with the group */
  bool conflict(const vcfl & line) const;  
  /** \brief solves conflict, remains clean struct*/
  vcfl solve();  
  /** \brief adds a line to the group */
  void add(const vcfl & line);   
    /** \brief clears object */
  void clear();   
  /** \brief if group is empty*/
  bool empty() const;   
  /** \brief generates the reference string of the conflict group */
  std::string generate_ref() const;  
  /** \brief generates alternatives and adds them to a list */
  void combine(const std::string & ref, const unsigned int & p, std::string & new_alts, unsigned int i = 0,unsigned int off = 0,  std::string tmpalt = "") const;  
  /** \brief applies all SNPS in the list to ref */
  void apply_snps(std::string & ref);  
  /** \brief returns true if all lines are SNPs */
  bool only_snps() const;  
};

std::ostream& operator <<(std::ostream& outputStream, const conflict_group& c);
#endif
