/**
	construct_jia.cpp
	Purpose: Counstuct jump-index-array

	@author Thomas Büchler
*/

#include "construct_jia.hpp"
#include <vector>

#include "../methods/construct_methods.hpp"//bitsneeded


using namespace std;
using namespace sdsl;


/** \brief default constructor*/
construct_jia::construct_jia(){
	indel_infos = deque<construct_jia::indel_info>();
	cnv_infos = deque<construct_jia::cnv_info>();
}

/** \brief adds an indel entry to the queue*/
void construct_jia::add_indel(bool in_sv, unsigned int rank_site, unsigned int rank_alleles_start, unsigned int allele_count){
	indel_infos.push_back(make_tuple(in_sv,rank_site,rank_alleles_start,allele_count));
}

/** \brief adds an cnv entry to the queue*/
void construct_jia::add_cnv(unsigned int rank_site_start, bit_vector copies, unsigned int rank_left_end){
	cnv_infos.push_back(make_tuple(rank_site_start,copies,rank_left_end,0));
}

/** \brief sets the rigth end of the last cnv entry*/
void construct_jia::cnv_set_rigth_end(unsigned int rank_rigth_end){	
	get<3>(cnv_infos.back()) = rank_rigth_end;
}

/** 
 * \brief constructs the jump index array
 */
jumpindexarray construct_jia::jia(unsigned int marker_count, unsigned int marker_jis_count, unsigned int marker_lay1_count){
	
	int bit_width = bitsneeded(marker_count);
	
	// vectors of jia
  vector<int_vector<0>> jumptargets	= vector<int_vector<0>>(marker_count);	
  bit_vector to_var 	= int_vector<1>(marker_count,0);
	
	// process indel_infos
	for(auto indel = indel_infos.begin(); indel!=indel_infos.end(); ++indel){ 
		
		// load infos
		unsigned int site_marker 			= get<1>(*indel);
		unsigned int allele_left_end 	= get<2>(*indel) + marker_jis_count + marker_lay1_count; // marker in tmp_lay2
		unsigned int allele_count 		= get<3>(*indel);		
		// site marker in sv
		if(get<0>(*indel))
			site_marker += marker_jis_count;
		

		//fill in values
		int_vector<0> targets = int_vector<0>(allele_count,0,bit_width);
		for(unsigned int i = 0; i < allele_count; i++){
			//jump from allele left end to site marker
			jumptargets[allele_left_end + i ] = int_vector<0>(1,site_marker,bit_width); 
			//jump from site marker to allele rigth end
			targets[i] = allele_left_end + i + 1;
		}
		jumptargets[site_marker] 	= move(targets);
		to_var[site_marker]				= true;		
	}
	
	
	// process cnv_infos
	for(auto cnv = cnv_infos.begin(); cnv != cnv_infos.end(); ++cnv){ 
		
		// load infos
		unsigned int site_marker_start 	= get<0>(*cnv);
		bit_vector copies 							= get<1>(*cnv);	//number of site markers is copies.size() - 1
		unsigned int sv_left_end	 			= get<2>(*cnv) + marker_jis_count; // marker in tmp_lay1
		unsigned int sv_rigth_end	 			= get<3>(*cnv) + marker_jis_count; // marker in tmp_lay1s
		
		//jump back from sv to site marker
		int_vector<0> site_markers = int_vector<0>(copies.size() - 1,0,bit_width); 
		for(long unsigned int i = 0; i < site_markers.size(); i++)
			site_markers[i] = site_marker_start + i;		
		jumptargets[sv_left_end] = move(site_markers);
		
		//jump to sv
		for(long unsigned int i = 0; i < copies.size()-1; i++){ 
			to_var[site_marker_start + i] = true;
			if(copies[copies.size() - i - 2]){ 
				//jump additionally to first site marker (== ignore rest of site markers)
				jumptargets[site_marker_start + i] = int_vector<0>(2,sv_rigth_end,bit_width);
				jumptargets[site_marker_start + i][0] = site_marker_start;
			}else 
				//jump only to sv sequence
				jumptargets[site_marker_start + i] = int_vector<0>(1,sv_rigth_end,bit_width);
		}			
	}
	
	return move(jumpindexarray(move(jumptargets),move(to_var)));
	
}
