/**
	construct_jia.hpp
	Purpose: Counstuct jump-index-array

	@author Thomas Büchler
*/

#include <tuple>        // std::tuple
#include <deque>
#include <sdsl/int_vector.hpp>

#include "jumpindexarray.hpp"

#ifndef CONJIA_J
#define CONJIA_J

/**
 * \brief this struct collects all jump information while the construction of jis  and caclulates jia out of this information 
 */
struct construct_jia{
	/**
	 * indel_jump<1> indicates whether the site marker lies in a sv
	 * indel_jump<2> is the rank of the site marker
	 * indel_jump<3> is the rank of the alle marker, where the alleles begin (left end)
	 * indel_jump<4> is number of alleles
	 */
	typedef std::tuple<bool,unsigned int, unsigned int, unsigned int>	indel_info;
	
	/**
	 * cnv_jump<1> is the rank of the first site marker
	 * cnv_jump<2> is the copies as in construct_jis::cnv::copies
	 * cnv_jump<3> is rank of left end marker of the sv
	 * cnv_jump<4> is rank of rigth end marker of the sv
	 */
	typedef std::tuple<unsigned int, sdsl::bit_vector, unsigned int, unsigned int> cnv_info;
	

	//queues for jump informations
	std::deque<indel_info> indel_infos;
	std::deque<cnv_info> 		cnv_infos;
	
	construct_jia();
	
	//add methods
	void add_indel(bool in_sv, unsigned int rank_site, unsigned int rank_alleles_start, unsigned int allele_count);
	void add_cnv	(unsigned int rank_site_start, sdsl::bit_vector copies, unsigned int rank_left_end);
	void cnv_set_rigth_end	(unsigned int rank_rigth_end);
	
	//construct_jia
	jumpindexarray jia(unsigned int marker_count, unsigned int marker_jis_count, unsigned int marker_lay1_count);	
};

#endif
