/**
	interval_list.hpp
	Purpose: Struct for "set of intervals" in backward search

	@author Thomas Büchler
*/
#include <string>
#include <list>
#include <fstream>

#include <sdsl/csa_wt.hpp>

#include "../structs/jumpindexarray.hpp"
#include "../structs/compressedjia_rank.hpp"
#include "../structs/searchinterval.hpp"

#ifndef IVL_H
#define IVL_H


/**
 * (in Master Theasis: Alg. 7.2 Backward search with structural variants -> Intervals)
 * 
 *  \brief struct for the set of intervals used in the matcher.
 * 
 * set implemented by std::list
 * 
 * interfaces: merge, erase, add, empty, begin, end 
 * 
 * implemented serialize and load
 * 
 */
struct interval_list{
    typedef sdsl::int_vector<>::size_type	size_type;
    typedef std::list<searchinterval>::iterator	iterator;
    typedef std::list<searchposition>::iterator	iterator_single;
    typedef std::list<searchinterval>::const_iterator	const_iterator;
    typedef std::list<searchposition>::const_iterator	const_iterator_single;
    typedef sdsl::csa_wt<sdsl::wt_huff<sdsl::bit_vector,sdsl::bit_vector::rank_1_type,sdsl::select_support_scan<1>,sdsl::select_support_scan<0>>,32,1024*1024>		fm_typ_sa_noisa;
    
    //data
    std::list<searchinterval> set;
    std::list<searchposition> singleset;

    
    /** \brief default constructor*/    
    interval_list();    
    
    /** manipulating methods*/
    void 	merge(interval_list & ivl);
    iterator 		erase(iterator 		position);
    iterator_single 	erase(iterator_single 	position);
    void 	add(const searchinterval& iv);
    void 	add(const searchposition& sp);
    
    /** status methods*/
    bool 	empty()	const;
    // for set
    iterator 	begin();
    iterator 	end();
    const_iterator 	begin() const;
    const_iterator 	end() 	const;    
    
    
    /** status methods singleset*/
    iterator_single 	begin_single();
    iterator_single 	end_single();
    const_iterator_single 	begin_single()	const;
    const_iterator_single 	end_single()	const;   
    
    
    /** output methods */
    unsigned int number_of_results();
    
    /** \brief serializes object with sdsl methods*/
    size_type	serialize(std::ostream& out, sdsl::structure_tree_node* v,std::string name) const;
    /** \brief loads serialized object with sdsl methods*/
    void	load(std::istream& in);    
};

std::ostream& operator <<(std::ostream& outputStream, const interval_list& ivl);


#endif
