/**
	jumpindexarray.hpp
	Purpose: Struct for jump-index-array

	@author Thomas Büchler
*/

#include <vector>

#include <sdsl/int_vector.hpp>

#ifndef JUM_H
#define JUM_H

/**
 * 
 * (In Master Thesis: 6.3.3 Sprungtabelle)
 * 
 *  \brief a struct containig all jump infomation needed for the 
 * 
 *  it contains a vector with a vector of targets for each # 
 *  and a bitvector that defines if an # jumps to a variation or to the reference string 
 */
struct jumpindexarray{
  
  //data
  std::vector<sdsl::int_vector<0>> 	jumptargets;
  sdsl::bit_vector			to_var;
  
  //constructors
	
  /** \brief copy constructor */
  jumpindexarray(const jumpindexarray & jia);  
  /** \brief copy constructor */
  jumpindexarray(std::vector<sdsl::int_vector<0>> jumptargets, sdsl::bit_vector	to_var);  
  /** \brief defaut constructor */
  jumpindexarray();
  
  //methods
  /** \brief returns a vector with the jumptargets*/
  sdsl::int_vector<0>	targets(unsigned int i);
  /** \brief returns if the jump goes to a variation*/
  bool 			to_vari(unsigned int i);  
  /** \brief cleans storage */
  void 			clear ();
	
	jumpindexarray get_transformed_jia(sdsl::int_vector<0> & permutation);
	jumpindexarray get_inverted_jia();
  
  //serializsation  
  /** \brief serealizes a jumpindex with sdsls serialize methods */
  typedef sdsl::int_vector<>::size_type	size_type;
  size_type	serialize(std::ostream& out, sdsl::structure_tree_node* v, std::string name) const;		
  /** \brief loads a serialized jumpindex with sdsls methods */
  void 		load(std::istream& in);    
};
 
std::ostream& operator <<(std::ostream& outputStream, const jumpindexarray& jia);

#endif
