/**
	kmer_index.hpp
	Purpose: Struct for kmer index

	@author Thomas Büchler
*/

#include <string>
#include <vector>

#include "../structs/interval_list.hpp"

#ifndef PRC_H
#define PRC_H


/** 
 * (In Master Thesis: 7.2 Verwendung eines k-mer-Index)
 *
 * \brief struct for precalculation of interval_list */
struct kmer_index{
  
  //data
  unsigned int k;  
  std::vector<interval_list> kmer;
  static const unsigned int letter_count = 4;	//A, T, G, C
  
  //constructors
  kmer_index();
  kmer_index(int k);
  kmer_index(int k, std::vector<interval_list>  ivl);
  kmer_index(int k, std::vector<interval_list> & ivl);
  //kmer_index(int k, std::vector<interval_list> * ivl);
  
  //methods
  static unsigned int		base2int(const char & c );
  static char						int2base(const unsigned int i);  
  static unsigned int		get_count(const std::string & s);
  static std::string	get_string(unsigned int count, const unsigned int k);
  interval_list 	get_ivlist(const std::string & s) const;
  void 			remove_n(const  int & n);
  
  
  //serialisation 
  typedef sdsl::int_vector<>::size_type size_type;
  size_type	serialize(std::ostream& out, sdsl::structure_tree_node* v,std::string name) const;
  void	load(std::istream& in);
  

};

#endif
