/**
	matchresult.cpp
	Purpose: Storing the information of the backward search

	@author Thomas Büchler
*/
#include "matchresult.hpp"

using namespace std;
using namespace sdsl;

/** \brief default constructor*/
matchresult::matchresult(){}

/** \brief add a searchresult to the list*/
void matchresult::add_res(const searchresult srs){
  match_res.push_back(move(srs));
}

/** \brief calculates how many reads are mapped*/
unsigned int matchresult::mapped(){
  unsigned int res = 0;
  for(auto srs = match_res.begin(); srs != match_res.end(); srs++)
    if(srs->ivl.number_of_results() > 0)	res++;

  return res;
}

/** \brief calculates how many reads are mapped twice (read + inverse)*/
unsigned int matchresult::twice_mapped(){
	bool inv = true;			//indicator whether current result belongs to inverse pattern
  bool norm_map = false;//indicator whether previos read mapped
  unsigned int twice_map  = 0;
	
  for(auto srs = match_res.begin(); srs != match_res.end(); srs++){
		inv = !inv; 		
		if(!inv){
			int rsc = srs->ivl.number_of_results();
			norm_map = rsc > 0;
		}else{
			if(!norm_map)
				continue;
			int rsc = srs->ivl.number_of_results();
			if(rsc > 0)
				twice_map++;
		}
	}
	return twice_map;
}

/** \brief calculates how many mapping results are found*/
unsigned int matchresult::results(){
  unsigned int res = 0;
  for(auto srs = match_res.begin(); srs != match_res.end(); srs++)
    res += srs->ivl.number_of_results();
  return res;
}

/** \brief serialize struct*/
matchresult::size_type matchresult::serialize(ostream& out, structure_tree_node* v, string name) const{   
  structure_tree_node *child = structure_tree::add_child(v, name, sdsl::util::class_name(*this));
  size_type written_bytes = 0;  
  
  int_vector<32> size_vec = int_vector<32>(1,match_res.size(),32);
  
  written_bytes += size_vec.serialize(out, child, "size");
  
  for(auto s_res = match_res.begin(); s_res != match_res.end(); ++s_res)
    written_bytes += s_res->serialize(out,child,"s_res");

  structure_tree::add_size(child, written_bytes);
  return written_bytes;  
}

/** \brief load serialized struct*/
void matchresult::load(istream& in){
  int_vector<32> size_vec;
  size_vec.load(in);
  
  match_res = list<searchresult> ();
  for(int i = 0; i < (int) size_vec[0]; i++){
    searchresult s_res;
    s_res.load(in);
    match_res.push_back(s_res);    
  }	
}
