/**
	position_helper.hpp
	Purpose: Linking jis posions to fasta posiontions, chromosoms and variations

	@author Thomas Büchler
*/
#include <sdsl/int_vector.hpp>
#include <deque>
#include <string>
#include <vector>
#include <ostream>

#ifndef POSHELP_H
#define POSHELP_H

/**
 * \brief struct that stores the connection between a marker and its reference position in the fasta file
 *
 * Let i be the rank of a marker in jis
 * * fa_pos[i] is the corresponding position in the fasta file
 * * 	more exacly: the base rigth before the marker derived from the base fasta[fa_pos[i]-1]
 * * vcf_id[i] is the id-string of the variation in the vcf file (for i is a site marker)
 * 
 */
struct position_helper{	
	
	/** reference positions of marker with jis rank i is ref_pos[i] */
	sdsl::int_vector<0> fa_pos;
	
	/** marker with jis rank i has id vcf_id[i] the vcf file. (for marker i is site marker) */
	std::vector<std::string> vcf_id;
	
	/** names of the including chromosmes */
	std::vector<std::string> chromosome_names;
	
	/** for i < chromosome_count=chromosome_names.size()
	* seq_seperator[i-1] is jis rank of marker between chromi and chromi+1 
	* seq_seperator[chromosome_count-1] is jis rank of marker between chrom and cnv section
	* seq_seperator[chromosome_count] is jis rank of marker between cnv section and var section */
	sdsl::int_vector<0> seq_seperator;
	
	//serialisation 
  typedef sdsl::int_vector<>::size_type	size_type;
  size_type	serialize(std::ostream& out, sdsl::structure_tree_node* v,std::string name) const;
  void		load(std::istream& in);  
};

/**
 * \brief struct that used to cunstruct position_helper
 */
struct construct_position_helper{	
	std::deque<unsigned int> fa_pos_jis;
	std::deque<unsigned int> fa_pos_tmp1;
	std::deque<unsigned int> fa_pos_tmp2;
	
	std::deque<std::string> chromosome_names;
	std::deque<std::string> vcf_id_jis;
	std::deque<std::string> vcf_id_tmp1;
	
	std::deque<unsigned int> seq_seperator;
	
	position_helper convert();
};

std::ostream& operator <<(std::ostream& outputStream, const position_helper& ph);
#endif
