/**
	searchresult.cpp
	Purpose: Storing the information of a search interval

	@author Thomas Büchler
*/

#include "searchinterval.hpp"
#include <sdsl/int_vector.hpp>


using namespace std;
using namespace sdsl;

/** \brief constructor */
searchinterval::searchinterval(searchinterval::bounds hlr, int hN): lr(hlr), N(hN){};

/** \brief serialize */
searchinterval::size_type searchinterval::serialize(ostream& out, sdsl::structure_tree_node* v, string name) const{
  sdsl::structure_tree_node *child = sdsl::structure_tree::add_child(v, name, sdsl::util::class_name(*this));
  searchinterval::size_type written_bytes = 0;
  
  int_vector<32> data_vec = int_vector<32>(3);    
  data_vec[0] 	= N;
  data_vec[1] 	= lr.first;
  data_vec[2] 	= lr.second;  
  
  written_bytes += data_vec.serialize(out, child, "searchinterval");

  sdsl::structure_tree::add_size(child, written_bytes);
  return written_bytes;
}

/** \brief serialize */
void searchinterval::load(istream& in){
  int_vector<32> data_vec;    
  data_vec.load(in);  
  N		= data_vec[0];
  unsigned l	= data_vec[1];
  unsigned r	= data_vec[2];
  lr 		= bounds(l,r);
}

/** \brief constructor */
searchposition::searchposition(unsigned int hpos, int hN , deque< unsigned int > hret):pos(hpos),N(hN),ret(hret){}

/** \brief serialize */
searchposition::size_type searchposition::serialize(ostream& out, structure_tree_node* v, string name) const{
  sdsl::structure_tree_node *child = sdsl::structure_tree::add_child(v, name, sdsl::util::class_name(*this));
  searchinterval::size_type written_bytes = 0;
  
  unsigned int stacksize = ret.size();
  unsigned int vectorsize = 2+stacksize;
  int_vector<32> sp_vec = int_vector<32>(vectorsize);
  
  sp_vec[0] 	= N;
  sp_vec[1] 	= pos;  
  for(unsigned int j = 0; j < stacksize; j++)
    sp_vec[j+2]	= ret[j];
  
  written_bytes += sp_vec.serialize(out, child, "searchpos_vec");

  sdsl::structure_tree::add_size(child, written_bytes);
  return written_bytes;
}

/** \brief serialize */
void searchposition::load(istream& in){
  
  int_vector<32> sp_vec;    
  sp_vec.load(in);
  unsigned int stacksize = sp_vec.size() - 2;
  
  N	= sp_vec[0];
  pos	= sp_vec[1];
  ret = deque<unsigned int>(stacksize);
  
  for(unsigned int j = 0; j < stacksize; j++)
    ret[j]	= sp_vec[j+2];
}
