/**
	searchresult.hpp
	Purpose: Storing the information of a search interval

	@author Thomas Büchler
*/

#include <iostream>
#include <stack>
#include <deque>
#include <tuple>
#include <utility>
#include <iomanip>      // std::setw

#include <sdsl/int_vector.hpp>

#ifndef SIV_H
#define SIV_H

/**
 * 
 * (In Master Thesis: 7.1.1 Grundlegender Algorithmus -> interval iv)
 * 
 * \struct Struct for representation of a sa interval
 * 
 * l left bound
 * r rigth bound
 * 
 * N number of mapped 'N'
 */
struct searchinterval{  
  typedef std::pair<unsigned int ,unsigned int> 	bounds;	//tuple of left and rigth bound
  
  //data  
  // the interval
  bounds lr;  
  // number of mapped N
  int N;
  
  // constructors
  searchinterval(bounds hlr, int hN = 0);
  
  //seralization  
  /** \brief serializes object with sdsl methods*/
  typedef sdsl::int_vector<>::size_type	size_type;
  size_type	serialize(std::ostream& out, sdsl::structure_tree_node* v,std::string name) const;
  /** \brief loads serialized object with sdsl methods*/
  void	load(std::istream& in);  
};

/**
 * \struct Struct for representation of a position for backward string search 
 * 
 * p position
 * 
 * N number of mapped 'N'  
 */
struct searchposition{  
  
  //data  
  // the position
  unsigned int pos;  
  // number of mapped N
  int N;
  // the return stack
  std::deque<unsigned int> ret;
  
  // constructors
  searchposition(unsigned int hpos, int hN = 0, std::deque<unsigned int> hret = std::deque<unsigned int>());
  
  //seralization  
  /** \brief serializes object with sdsl methods*/
  
  typedef sdsl::int_vector<>::size_type	size_type;  
  size_type	serialize(std::ostream& out, sdsl::structure_tree_node* v,std::string name) const;
  /** \brief loads serialized object with sdsl methods*/
  void	load(std::istream& in);   
};

#endif
