/**
	searchresult.cpp
	Purpose: Storing the information of the backward search

	@author Thomas Büchler
*/
#include "searchresult.hpp"
#include <stdexcept>	// invalid_argument

using namespace std;
using namespace sdsl;

/** \brief default constructor*/
searchresult::searchresult(){}

/** \brief copy constructor*/
searchresult::searchresult(string hname, string hpattern, string hqual, interval_list hivl): name(hname), pattern(hpattern), qual(hqual), ivl(hivl){};

/** \brief transforms a int_vector to a string*/
string searchresult::vec2string(const int_vector< int(8) >& v){
  string s	= string(v.size(), 'x');
  for(int i = 0; i < (int) s.size(); i++)
    s[i]	= (char) (short int) v[i];
  
  return move(s);
}

/** \brief transforms a string to a int_vector*/
int_vector< int(8) > searchresult::string2vec(const string& s){
  int_vector<8> iv = int_vector<8>(s.size());
  for(int i = 0; i < (int) s.size(); i++)
    iv[i]	= s[i];
  
  return move(iv);
}

/** \brief serialize struct*/
searchresult::size_type searchresult::serialize(ostream& out, sdsl::structure_tree_node* v, string name) const{
  sdsl::structure_tree_node *child = sdsl::structure_tree::add_child(v, name, sdsl::util::class_name(*this));
  size_type written_bytes = 0;
  
  written_bytes += searchresult::string2vec(this->name)	.serialize(out, child, "name");
  written_bytes += searchresult::string2vec(pattern)	.serialize(out, child, "pattern");
  written_bytes += searchresult::string2vec(qual)	.serialize(out, child, "qual");
  written_bytes += ivl					.serialize(out, child, "interval_list");

  sdsl::structure_tree::add_size(child, written_bytes);
  return written_bytes;
}

/** \brief load serialized struct*/
void searchresult::load(istream& in){
  int_vector<8> v;
  v.load(in);
  name 		= searchresult::vec2string(v);
  v.load(in);
  pattern 	= searchresult::vec2string(v);
  v.load(in);
  qual	 	= searchresult::vec2string(v);
  
  ivl.load(in);	
}


