/**
	sequence_streams.cpp
	Purpose: Storing information about the streams to vcf-files, jis-file and tmp-files

	@author Thomas Büchler
*/

#include <string>
#include <fstream> //ifstream, ofstream

#ifndef SEQSTREAMS_H
#define SEQSTREAMS_H

/**
 * \brief this struct contains all information about the streams to the files with sequence data
 * 
 * 
 */
struct sequence_streams{
	//data  
	//filenames
	std::string	jis_filename;
	std::string	snp_filename;
	std::string	ind_filename;
	std::string	sv_filename;
	std::string	fa_filename;
	std::string	tmp_lay1_filename;
	std::string	tmp_lay2_filename;
	
	
	//input streams
  std::ifstream	fa_in;   
  std::ifstream	snp_in;
  std::ifstream	ind_in;
  std::ifstream	sv_in;
  std::ifstream	tmp_lay1_in;
  std::ifstream	tmp_lay2_in;
	
	
	//output streams
  std::ofstream	snp_out;
  std::ofstream	ind_out;
	std::ofstream	sv_out;   
  std::ofstream	jis_out;
  std::ofstream	tmp_lay1_out;
  std::ofstream	tmp_lay2_out;
	
	//counters
	unsigned int jis_markers;
	unsigned int lay1_markers;
	unsigned int lay2_markers;	
	unsigned int current_fa_pos;
	std::string fa_chrom;
	
	//contol
	bool write_in_jis; //if false: write in tmp_lay1_out till pos write_in_lay1_end
	bool write_in_lay1_ended; //if true stream was set back to jis
	unsigned int last_chrom_length;
	unsigned int write_in_lay1_end;
	
	
	//constructors
	sequence_streams();
	sequence_streams(std::string & fa_filename, std::string & vcf_dirname, std::string out_dirname);
	
	//methods
	void close();
	void open_for_jis();
	void open_for_prepvcf(std::string & vcf_filename);
	
  /** \brief copy the chars in fa file to jis till pos is reached*/
	void copy_till_pos(unsigned int pos);
	
	/** \brief copy the chars in fa file to jis till end is reached*/
	void copy_till_end();
	
	/** \brief writes char to jis */
	void write_char(char c);
	
	/** \brief writes marker jis */
	void write_marker();
	
	/** \brief writes an alternative to tmp_lay2_out */
	void write_alternative(std::string & alt);
	
	/** \brief skippes to the next chrom (maked by '>') in the .fa file and copies the bases to the .jis file*/
	void copy_till_next_chrom();
	
	/** \brief check if reference matches the string */
	bool check_ref(std::string & s);
	bool check_ref(char & c);
	
	void check_lay1_ended();
	
};
	
#endif
