/**
	transform.hpp
	Purpose: Store information to easily convert diffrent numberings of the markers

	@author Thomas Büchler
*/

#include <map>
#include <sdsl/int_vector.hpp>
#include "../methods/construct_methods.hpp"

#ifndef TID_H
#define TID_H

/** 
 * 
 * (In Master Thesis: 6.5 Transformation der Nummerierungen)
 * 
 * \brief  struct for transform hash-id's
 *
 * contains vectors/maps to transform the id of a hash in another id.
 *
 * id's are:
 * ps: position in jump index string (jis)
 * rs: rank in jis
 * rb: rank in bwt of jis  
*/
struct transform_id{
  
  //primary data
  sdsl::int_vector<0> rb2ps;	//rank bwt -> pos  seq
  unsigned int seqsize;
  //calculated data
  std::map<unsigned int,unsigned int> ps2rb;	//pos  seq -> rank bwt   
  sdsl::int_vector<0> rs2rb;	//rank seq -> rank bwt
  sdsl::int_vector<0> rb2rs;	//rank bwt -> rank seq
  sdsl::int_vector<0> rs2ps;	//rank seq -> pos  seq  
  //other transform methods
	unsigned int ps2rs(unsigned int ps);
	
	

	unsigned int next_marker(unsigned int pos);

  

  // constructors  
  transform_id();
  
  /** \brief constructor that calculates everything by he fm_index*/
  template <class fm_typ>
  static transform_id factory(const fm_typ& fm_index){
    using namespace sdsl;
    
    transform_id tr = transform_id();    
    tr.seqsize = fm_index.size();
    
    unsigned int hash_C_pos 	= fm_index.C[ fm_index.char2comp['#']];
    unsigned int numberOfHash 	= fm_index.C[ fm_index.char2comp['#']+1]- hash_C_pos;
    
    //calc rb2ps
    tr.rb2ps = int_vector<0>(numberOfHash,0,bitsneeded(tr.seqsize));

    //loop over rank and lookup position
    for(unsigned int  hash_bwt_rank = 0; hash_bwt_rank < numberOfHash; hash_bwt_rank++)
      tr.rb2ps[hash_bwt_rank] = fm_index[hash_C_pos+hash_bwt_rank];    
    
    //calc others
    tr.make_vectors();
    return std::move(tr);
  }; 
  
  //serialisation 
  typedef sdsl::int_vector<>::size_type	size_type;
  size_type	serialize(std::ostream& out, sdsl::structure_tree_node* v,std::string name) const;
  void		load(std::istream& in);  
    
private: //methods
  void make_vectors();
  std::map<unsigned int,unsigned int>	make_ps2rb();
  sdsl::int_vector<0> 	make_rs2rb();
  sdsl::int_vector<0> 	make_rs2ps();
};

#endif
