/**
	vcfl.cpp
	Purpose: Representation of a vcf line

	@author Thomas Büchler
*/

#include "../structs/vcfl.hpp"
#include "../methods/iupac_meth.hpp"

#include <iostream>

using namespace std;
/** \brief calculates queue of alternatives */
deque<string> vcfl::genaltQ() const{
  deque<string> altQ;
  int alt_begin 	= 0;
  int alt_length	= 0;


  for(unsigned int i = 0; i < alt.size(); i++)   
    if(alt[i] != ',')	alt_length++; 	
    else{    //if "," add substring to altQ          
      altQ.push_back(alt.substr(alt_begin,alt_length));       
      alt_begin 	= i+1;      
      alt_length 	= 0;     
    }
    
  altQ.push_back(alt.substr(alt_begin,alt_length));
	return move(altQ);
}


  /** \brief parameterized constructor
vcfl::vcfl(const int &hpos, const string &href, const string &halt, const string &hchrom, const int hline)
  : pos(hpos), ref(href),alt(halt),chrom(hchrom),line(hline){//copy entries  
  //calculate alt queue
  genaltQ();    
};*/

vcfl::vcfl(const string &chrom, const int pos, const string &id, const string & ref, const string & alt, const string & info)
:chrom(chrom),pos(pos),id(id),ref(ref),alt(alt),info(info){}


  /** \brief parameterized constructor
vcfl::vcfl(const int &hpos, const string &href, const deque<string> & haltQ, const string &hchrom, const int hline)
  : pos(hpos), ref(href),altQ(haltQ),chrom(hchrom),line(hline){//copy entries    
  //calculate alt string
  alt = altQ[0];  
  for(unsigned int i = 1; i < altQ.size(); i++){//firts alt is set bevor
    alt += "," + altQ[i];
  }     
};*/

/** \brief parses VCF line entry read from file*/
vcfl::vcfl(const string & line){//, const unsigned int linecount):line(linecount){
	unsigned int i 	= 0;
  unsigned int chromend	= 0;
  unsigned int idbegin	= 0;
  unsigned int idend 		= 0;
  unsigned int posbegin	= 0;
  unsigned int posend 	= 0;
  unsigned int refbegin	= 0;
  unsigned int refend 	= 0;
  unsigned int altbegin	= 0;
  unsigned int altend 	= 0;
	unsigned int infobegin= 0;
  unsigned int infoend 	= 0;
    //parse the line      
  while(!isspace(line[i])		&& i < line.length()) i++; 	//i = chromend
  chromend	= i;  
	
  while( isspace(line[i])		&& i < line.length()) i++;	//i = posbegin
  posbegin	= i;
  while(!isspace(line[i])		&& i < line.length()) i++; 	//i = posend  
  posend	= i;
	
  while( isspace(line[i])		&& i < line.length()) i++;	//i = idbegin
  idbegin	= i;
  while(!isspace(line[i]) 	&& i < line.length()) i++; 	//i = idend  
  idend = i;
  
  while( isspace(line[i]) 	&& i < line.length()) i++;	//i = refbegin
  refbegin	= i;
  while(!isspace(line[i]) 	&& i < line.length()) i++; 	//i = refend 
  refend	= i;  
	
  while( isspace(line[i]) 	&& i < line.length()) i++;	//i = altbegin
  altbegin	= i;
  while(!isspace(line[i]) 	&& i < line.length()) i++; 	//i = altend
  altend	= i;    
	
	while( isspace(line[i])		&& i < line.length()) i++;	//i = QUALbegin
  while(!isspace(line[i]) 	&& i < line.length()) i++; 	//i = QUALend  
  
	while( isspace(line[i])		&& i < line.length()) i++;	//i = FILTERbegin
  while(!isspace(line[i]) 	&& i < line.length()) i++; 	//i = FILTERend  
  
	while( isspace(line[i])		&& i < line.length()) i++;	//i = INFObegin
	infobegin = i;
  while(!isspace(line[i]) 	&& i < line.length()) i++; 	//i = INFOend 
	infoend = i;



  //	initalise data
  chrom = line.substr(0,chromend);
  id		= line.substr(idbegin,idend-idbegin);
  ref		= line.substr(refbegin,refend-refbegin);
  alt 	= line.substr(altbegin,altend-altbegin);  
  info	= line.substr(infobegin,infoend-infobegin); 
  pos		= stoi(line.substr(posbegin,posend-posbegin)); 
}


vcfl::vcfl(){}


/**
 * \brief checks if a vcf line is a SNP 
 */
bool vcfl::isSNP() const{
    //ref must be one base
    if (ref.size()>1)
      return false;
    
    int altlength = 0;
    for(unsigned int i = 0; i < alt.size(); i++){
      altlength++;
      
      // with ',' the next alternative starts
      if(alt[i] == ',')
	altlength = 0;
      
      //length of each alternative must be one
      if (altlength>1)
	return false;    
    }
    return true;  
 }
 
	/**
	* \brief checks if an INDEL or SNP (if alternative is specified)
	*/
	bool vcfl::isINDEL() const{
		return alt[0]!='<';
	}
	
	
	/**
	 * \brief checks wheater a vcf line is equal to this line
	 */
	bool vcfl::equal(vcfl & buf) const{
		return(pos == buf.pos && ref == buf.ref && alt == buf.alt && chrom == buf.chrom);
	}
	
	bool vcfl::empty() const{
		return (pos == 0 && chrom == "");		
	}

	/** \brief returns number of commas in alt string +1 */
  int vcfl::altCount() const{
		unsigned int out = 1;    
			for(unsigned int i = 0; i < alt.size(); i++)
				if(alt[i] == ',') out++;
		return out;
	}
	
	/** \brief returns max copy number */
  int vcfl::maxCN() const{
		//alt has the form <CNx1>,<CNx2>,... with integers x1,x2..
		//reads numbers between 'N' and '>' and 
		size_t x_pos2 = 0;
		int max_x = 1; //ref occours 1 time
		int ac = altCount();
		for(int i = 0; i < ac; i++){
			size_t x_pos1 = alt.find('N',x_pos2)+1;
			x_pos2 = alt.find('>',x_pos1);
			int x = stoi(alt.substr(x_pos1,x_pos2-x_pos1));
			if(x > max_x) max_x=x;
		}	
		return max_x;
	}
	
	
	/**
	* \brief count how many markers are needed for this variation
	*
  unsigned int vcfl::neededMarkers() const{		
	
		//SNP
		if(isSNP()) return 0;
		
		//INDEL
		if(isINDEL()){ //count ','			
			return altCount() + 2; // +1 for ref, +1 for locus marker 
		}
		
		//SV
		if(alt.substr(0,3)!="<CN"){ //currently only CNV will be processed
			return 0;
		}		
		
		
		//alts has the form <CNx1>,<CNx2>,... with integers x1,x2..
		//search for N and > and read number between
		size_t x_pos2 = 0;
		int max_x = 1; //ref occours 1 time
	
		for(int i = 0; i < altCount(); i++){
			size_t x_pos1 = alt.find('N',x_pos2)+1;
			x_pos2 = alt.find('>',x_pos1);
			int x = stoi(alt.substr(x_pos1,x_pos2-x_pos1));
			if(x > max_x) max_x=x;
		}	
		return max_x+1; // +1 for variant marker		
	}
*/

	
/**
 * \brief searches for ";END=x;" in info string and returns x
 */
unsigned int vcfl::sv_end() const{
	size_t end_pos1 = info.find(";END=")+5;
	size_t end_pos2 = info.find(';',end_pos1);
	return stoi(info.substr(end_pos1,end_pos2-end_pos1));
}

/** \brief defiening an string for a variation for streams, for debugging*/
ostream& operator <<(ostream& outputStream, const vcfl& v){    
	if(v.isINDEL())
		outputStream << v.chrom << '\t' << v.pos << '\t' << v.id << '\t' << v.ref << '\t' << v.alt << "\t*\t*\t*\n";
	else
		outputStream << v.chrom << '\t' << v.pos << '\t' << v.id << '\t' << v.ref << '\t' << v.alt << "\t*\t*\t" << v.info << endl;
	return outputStream;
}


