/**
	vcfl.hpp
	Purpose: Representation of a vcf line

	@author Thomas Büchler
*/

#include <string>
#include <ostream>
#include <deque>

#ifndef VCFL_H
#define VCFL_H

/**
 * (In Master Thesis: 2.3.3 VCF)
 * \brief this struct contains the data of a vcf line
 */
struct vcfl{    
  //data
  std::string	chrom;   
  unsigned int	pos;
  std::string	id;
  std::string	ref;
  std::string	alt;  
  std::string	info;   
  //unsigned int line; //line in the vcf file 
  
  //constructors:
  
  /** \brief parameterized constructor*/ 
  vcfl(const std::string &chrom, const int pos, const std::string &id, const std::string & ref, const std::string & alt, const std::string & info);  
	
  //vcfl(const int &hpos, const std::string &href, const std::deque<std::string> & haltQ, 	const std::string &hchrom, const int hline);  
  /** \brief input parsing constructor*/
  vcfl(const std::string & line);//, const unsigned int linecount = 0);
  /** \brief default constructor*/   
  vcfl();
	
	std::deque<std::string> genaltQ() const;
  
  /** \brief true if the line encodes a SNP */
  bool isSNP() const;
	
	/** \brief checks if an INDEL or SNP (if alternative is specified)*/
	bool isINDEL() const;
  
	/** \brief count how many markers are needed for this variation*/
  //unsigned int neededMarkers() const;
	
	/** \brief checks wheater a vcf line is equal to this line */
	bool equal(vcfl & buf) const;
	
	/** \brief checks wheater a vcf line is the empty struct */
	bool empty() const;

	  /** \brief returns number of commas in alt string +1 */
  int altCount() const;
	
	  /** \brief returns max copy number*/
	int maxCN() const;
	
	/** \brief searches for ";END=x;" in info string and returns x */
	unsigned int sv_end() const;

};

std::ostream& operator <<(std::ostream& outputStream, const vcfl& v);
#endif
